#include <avr/io.h>
#include <stdio.h>
#include <avr/interrupt.h>

#include "uartdrv.h"



#define BUFFER_SIZE 128
volatile uint8_t buffer[BUFFER_SIZE+5];
volatile uint8_t bufferReadIdx = 0;
volatile uint8_t bufferWriteIdx = 0;

void uartdrvInit() {

  // UART
  //      TXD
  DDRD |= (1 << PD1);

  //      transmitter enable
  UCSRB = (1 << TXEN);
  //        8 data bit, no parity, 1 stop bit
  UCSRC = (1 << UCSZ1) | (1 << UCSZ0);
  //        2400 Baud @ 16MHz
  UBRRL = 0xa0;
  UBRRH = 0x01;;

  FILE *mystdout = fdevopen(uartdrvPutchar, NULL);
  stdout = mystdout;
}

void uartdrvWrite(uint8_t o) {
//  loop_until_bit_is_set(UCSRA, UDRE);
//  UDR = o;
  if (bufferWriteIdx == BUFFER_SIZE - 1) {
    while (bufferReadIdx == BUFFER_SIZE);
  } else {
    while (bufferReadIdx == bufferWriteIdx + 1);
  }
  buffer[bufferWriteIdx] = o;
  bufferWriteIdx++;
  UCSRB |= (1 << UDRIE);
}


int uartdrvPutchar(char c, FILE *stream) {
  if (c == '\n')
    uartdrvPutchar('\r', stream);
  uartdrvWrite((uint8_t) c);
  return 0;
}


ISR(USART_UDRE_vect) {
  if (bufferReadIdx != bufferWriteIdx) {
    UDR = buffer[bufferReadIdx];
    bufferReadIdx++;
    if (bufferReadIdx > BUFFER_SIZE) {
      bufferReadIdx = 0;
    }
  } else {
    UCSRB &= ~(1 << UDRIE);
  }
}
